@@ -1,10 +1,10 @@
 require "English"
+require "optparse"
 
 require "test/unit/color-scheme"
 require "test/unit/priority"
 require "test/unit/attribute-matcher"
 require "test/unit/testcase"
-require "optparse"
 
 module Test
   module Unit
@@ -15,7 +15,8 @@ module Test
       PREPARE_HOOKS = []
 
       class << self
-        def register_runner(id, runner_builder=Proc.new)
+        def register_runner(id, runner_builder=nil, &block)
+          runner_builder ||= Proc.new(&block)
           RUNNERS[id] = runner_builder
           RUNNERS[id.to_s] = runner_builder
         end
@@ -33,7 +34,8 @@ module Test
           @@default_runner = id
         end
 
-        def register_collector(id, collector_builder=Proc.new)
+        def register_collector(id, collector_builder=nil, &block)
+          collector_builder ||= Proc.new(&block)
           COLLECTORS[id] = collector_builder
           COLLECTORS[id.to_s] = collector_builder
         end
@@ -46,11 +48,13 @@ module Test
           ColorScheme[id] = scheme
         end
 
-        def setup_option(option_builder=Proc.new)
+        def setup_option(option_builder=nil, &block)
+          option_builder ||= Proc.new(&block)
           ADDITIONAL_OPTIONS << option_builder
         end
 
-        def prepare(hook=Proc.new)
+        def prepare(hook=nil, &block)
+          hook ||= Proc.new(&block)
           PREPARE_HOOKS << hook
         end
 
@@ -139,6 +143,9 @@ module Test
       attr_accessor :default_test_paths
       attr_accessor :pattern, :exclude, :base, :workdir
       attr_accessor :color_scheme, :listeners
+      attr_writer :stop_on_failure
+      attr_writer :debug_on_failure
+      attr_writer :gc_stress
       attr_writer :runner, :collector
 
       def initialize(standalone)
@@ -153,6 +160,9 @@ module Test
         @default_arguments = []
         @workdir = nil
         @listeners = []
+        @stop_on_failure = false
+        @debug_on_failure = false
+        @gc_stress = false
         config_file = "test-unit.yml"
         if File.exist?(config_file)
           load_config(config_file)
@@ -162,6 +172,14 @@ module Test
         yield(self) if block_given?
       end
 
+      def stop_on_failure?
+        @stop_on_failure
+      end
+
+      def debug_on_failure?
+        @debug_on_failure
+      end
+
       def prepare
         PREPARE_HOOKS.each do |handler|
           handler.call(self)
@@ -183,7 +201,7 @@ module Test
       def options
         @options ||= OptionParser.new do |o|
           o.banner = "Test::Unit automatic runner."
-          o.banner << "\nUsage: #{$0} [options] [-- untouched arguments]"
+          o.banner += "\nUsage: #{$0} [options] [-- untouched arguments]"
 
           o.on("-r", "--runner=RUNNER", RUNNERS,
                "Use the given RUNNER.",
@@ -236,57 +254,61 @@ module Test
 
           o.on("-n", "--name=NAME", String,
                "Runs tests matching NAME.",
-               "Use '/PATTERN/' for NAME to use regular expression.") do |name|
-            name = (%r{\A/(.*)/\Z} =~ name ? Regexp.new($1) : name)
+               "Use '/PATTERN/' for NAME to use regular expression.",
+               "Regular expression accepts options.",
+               "Example: '/taRget/i' matches 'target' and 'TARGET'") do |name|
+            name = prepare_name(name)
             @filters << lambda do |test|
-              return true if name === test.method_name
-              test_name_without_class_name = test.name.gsub(/\(.+?\)\z/, "")
-              if test_name_without_class_name != test.method_name
-                return true if name === test_name_without_class_name
-              end
-              false
+              match_test_name(test, name)
             end
           end
 
           o.on("--ignore-name=NAME", String,
                "Ignores tests matching NAME.",
-               "Use '/PATTERN/' for NAME to use regular expression.") do |n|
-            n = (%r{\A/(.*)/\Z} =~ n ? Regexp.new($1) : n)
-            case n
-            when Regexp
-              @filters << proc {|t| n =~ t.method_name ? false : true}
-            else
-              @filters << proc {|t| n != t.method_name}
+               "Use '/PATTERN/' for NAME to use regular expression.",
+               "Regular expression accepts options.",
+               "Example: '/taRget/i' matches 'target' and 'TARGET'") do |name|
+            name = prepare_name(name)
+            @filters << lambda do |test|
+              not match_test_name(test, name)
             end
           end
 
           o.on("-t", "--testcase=TESTCASE", String,
                "Runs tests in TestCases matching TESTCASE.",
-               "Use '/PATTERN/' for TESTCASE to use regular expression.") do |n|
-            n = (%r{\A/(.*)/\Z} =~ n ? Regexp.new($1) : n)
+               "Use '/PATTERN/' for TESTCASE to use regular expression.",
+               "Regular expression accepts options.",
+               "Example: '/taRget/i' matches 'target' and 'TARGET'") do |name|
+            name = prepare_name(name)
             @filters << lambda do |test|
-              match_test_case_name(test, n)
+              match_test_case_name(test, name)
             end
           end
 
           o.on("--ignore-testcase=TESTCASE", String,
                "Ignores tests in TestCases matching TESTCASE.",
-               "Use '/PATTERN/' for TESTCASE to use regular expression.") do |n|
-            n = (%r{\A/(.*)/\Z} =~ n ? Regexp.new($1) : n)
+               "Use '/PATTERN/' for TESTCASE to use regular expression.",
+               "Regular expression accepts options.",
+               "Example: '/taRget/i' matches 'target' and 'TARGET'") do |name|
+            name = prepare_name(name)
             @filters << lambda do |test|
-              not match_test_case_name(test, n)
+              not match_test_case_name(test, name)
             end
           end
 
           o.on("--location=LOCATION", String,
                "Runs tests that defined in LOCATION.",
-               "LOCATION is one of PATH:LINE, PATH or LINE") do |location|
-            if /\A\d+\z/ =~ location
+               "LOCATION is one of PATH:LINE, PATH or LINE.") do |location|
+            case location
+            when /\A(\d+)\z/
               path = nil
-              line = location.to_i
+              line = $1.to_i
+            when /:(\d+)\z/
+              path = $PREMATCH
+              line = $1.to_i
             else
-              path, line, = location.split(/:(\d+)/, 2)
-              line = line.to_i unless line.nil?
+              path = location
+              line = nil
             end
             add_location_filter(path, line)
           end
@@ -342,7 +364,7 @@ module Test
           end
 
           o.on("--config=FILE",
-               "Use YAML fomat FILE content as configuration file.") do |file|
+               "Use YAML format FILE content as configuration file.") do |file|
             load_config(file)
           end
 
@@ -361,6 +383,24 @@ module Test
             assertion_message_class.max_diff_target_string_size = size
           end
 
+          o.on("--[no-]stop-on-failure",
+               "Stops immediately on the first non success test",
+               "(#{@stop_on_failure})") do |boolean|
+            @stop_on_failure = boolean
+          end
+
+          o.on("--[no-]debug-on-failure",
+               "Run debugger if available on failure",
+               "(#{AssertionFailedError.debug_on_failure?})") do |boolean|
+            AssertionFailedError.debug_on_failure = boolean
+          end
+
+          o.on("--[no-]gc-stress",
+               "Enable GC.stress only while each test is running",
+               "(#{@gc_stress})") do |boolean|
+            @gc_stress = boolean
+          end
+
           ADDITIONAL_OPTIONS.each do |option_builder|
             option_builder.call(self, o)
           end
@@ -405,7 +445,7 @@ module Test
             n = 1
           end
           i += 1
-          keyword.sub(/^(.{#{n}})([A-Za-z]+)(?=\w*$)/, '\\1[\\2]')
+          keyword.sub(/^(.{#{n}})([A-Za-z-]+)(?=\w*$)/, '\\1[\\2]')
         end.join(", ")
       end
 
@@ -419,6 +459,12 @@ module Test
         @runner_options[:color_scheme] ||= @color_scheme
         @runner_options[:listeners] ||= []
         @runner_options[:listeners].concat(@listeners)
+        if @stop_on_failure
+          @runner_options[:listeners] << StopOnFailureListener.new
+        end
+        if @gc_stress
+          @runner_options[:listeners] << GCStressListener.new
+        end
         change_work_directory do
           runner.run(suite, @runner_options).passed?
         end
@@ -440,7 +486,7 @@ module Test
           if key == :arguments
             @default_arguments.concat(value.split)
           else
-            runner_options[key.to_sym] = value
+            runner_options[key] = value
           end
         end
         @runner_options = @runner_options.merge(runner_options)
@@ -480,6 +526,31 @@ module Test
         end
       end
 
+      def prepare_name(name)
+        case name
+        when /\A\/(.*)\/([imx]*)\z/
+          pattern = $1
+          options_raw = $2
+          options = 0
+          options |= Regexp::IGNORECASE if options_raw.include?("i")
+          options |= Regexp::MULTILINE if options_raw.include?("m")
+          options |= Regexp::EXTENDED if options_raw.include?("x")
+          Regexp.new(pattern, options)
+        else
+          name
+        end
+      end
+
+      def match_test_name(test, pattern)
+        return true if pattern === test.method_name
+        return true if pattern === test.local_name
+        if pattern.is_a?(String)
+          return true if pattern === "#{test.class}##{test.method_name}"
+          return true if pattern === "#{test.class}##{test.local_name}"
+        end
+        false
+      end
+
       def match_test_case_name(test, pattern)
         test.class.ancestors.each do |test_class|
           break if test_class == TestCase
@@ -504,6 +575,28 @@ module Test
                                    :method_name => test.method_name)
         end
       end
+
+      class StopOnFailureListener
+        def attach_to_mediator(mediator)
+          mediator.add_listener(TestResult::FINISHED) do |result|
+            result.stop unless result.passed?
+          end
+        end
+      end
+
+      class GCStressListener
+        def attach_to_mediator(mediator)
+          mediator.add_listener(TestCase::STARTED) do |test|
+            GC.start
+            GC.stress = true
+          end
+
+          mediator.add_listener(TestCase::FINISHED) do |test|
+            GC.start
+            GC.stress = false
+          end
+        end
+      end
     end
   end
 end
