@@ -61,13 +61,18 @@ module Test
         entry.start_with?("kernel/")
       end
 
+      def internal_backtrace_entry?(entry)
+        entry.start_with?("<internal:")
+      end
+
       def normalize_location(location)
         filtered_location = location.reject do |entry|
           jruby_backtrace_entry?(entry) or
-            rubinius_backtrace_entry?(entry)
+            rubinius_backtrace_entry?(entry) or
+            internal_backtrace_entry?(entry)
         end
         filtered_location.collect do |entry|
-          entry.sub(/:\d+:/, ":0:")
+          entry.sub(/:\d+:in [`'](?:[^']+?[#.])?/, ":0:in '")
         end
       end
 
@@ -100,8 +105,8 @@ module Test
                          :message   => "failure",
                          :test_name => "test_failure(TC_FailureError)",
                          :location  => [
-                           "#{__FILE__}:0:in `test_failure'",
-                           "#{__FILE__}:0:in `#{__method__}'",
+                           "#{__FILE__}:0:in 'test_failure'",
+                           "#{__FILE__}:0:in '#{__method__}'",
                          ],
                        },
                      ],
@@ -146,9 +151,9 @@ module Test
                          :message   => "nested",
                          :test_name => "test_nested_failure(TC_FailureError)",
                          :location  => [
-                           "#{__FILE__}:0:in `nested'",
-                           "#{__FILE__}:0:in `test_nested_failure'",
-                           "#{__FILE__}:0:in `#{__method__}'",
+                           "#{__FILE__}:0:in 'nested'",
+                           "#{__FILE__}:0:in 'test_nested_failure'",
+                           "#{__FILE__}:0:in '#{__method__}'",
                          ],
                        },
                      ],
@@ -193,9 +198,9 @@ module Test
           }
         end
         location = []
-        location << "#{__FILE__}:0:in `/'" if cruby?
-        location << "#{__FILE__}:0:in `test_error'"
-        location << "#{__FILE__}:0:in `#{__method__}'"
+        location << "#{__FILE__}:0:in '/'" if cruby?
+        location << "#{__FILE__}:0:in 'test_error'"
+        location << "#{__FILE__}:0:in '#{__method__}'"
         assert_equal([
                        {
                          :class     => Error,
@@ -546,6 +551,7 @@ module Test
         test_case = Class.new(TestCase) do
           data("label" => "value")
           def test_without_parameter
+            assert_equal("value", data)
           end
         end
 
@@ -554,8 +560,8 @@ module Test
                      suite.tests.collect {|test| test.method_name})
         result = TestResult.new
         suite.run(result) {}
-        assert_equal("1 tests, 0 assertions, 0 failures, " +
-                     "0 errors, 0 pendings, 0 omissions, 1 notifications",
+        assert_equal("1 tests, 1 assertions, 0 failures, " +
+                     "0 errors, 0 pendings, 0 omissions, 0 notifications",
                      result.summary)
       end
 
