@@ -1,9 +1,17 @@
+require "objspace"
+require "tempfile"
+require "bigdecimal"
+
 module TestUnitTestUtil
   private
+  def jruby?
+    RUBY_PLATFORM == "java"
+  end
+
   def jruby_only_test
-    begin
+    if jruby?
       require "java"
-    rescue LoadError
+    else
       omit("test for JRuby")
     end
   end
@@ -19,4 +27,9 @@ module TestUnitTestUtil
     test.run(result) {}
     result
   end
+
+  def fixture_file_path(file_name)
+    base_dir = File.dirname(__FILE__)
+    File.join(base_dir, "fixtures", file_name)
+  end
 end
